"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApp = void 0;
const get_fn_context_1 = require("./get-fn-context");
const utils_1 = require("./utils");
const builtin_method_1 = require("./builtin-method");
const is_method_enabled_1 = require("./is-method-enabled");
function createApp(options) {
    return async function main(params, context) {
        try {
            const fnContext = get_fn_context_1.getFnContext({ dsConfig: options.dsConfig, params, rawContext: context });
            if (!is_method_enabled_1.isMethodEnabled(params.methodName, options, params.forceCall)) {
                throw new Error(`method ${params.methodName} not found in datasource, or not enabled in this datasource`);
            }
            let fnParams = params.params || {};
            const isDataBase = options.dsConfig && options.dsConfig.type === 'database';
            // @ts-ignore
            const viewId = params.viewId || (isDataBase && options.dsConfig.schema['x-viewId']);
            const method = options.handlers[params.methodName];
            if ((method === null || method === void 0 ? void 0 : method.isHttp) && utils_1.isObj(params, 'defaultParams')) {
                fnParams = utils_1.mergeDeep(params.defaultParams, params.params || {});
            }
            // 内建数据源调用weda后台实现
            if (isDataBase) {
                if (builtin_method_1.isBuiltinMethod(params.methodName)) {
                    const builtinResult = await builtin_method_1.callBuiltinMethod(Object.assign(Object.assign({}, params), { viewId, 
                        // @ts-ignore
                        dataSourceName: options.dsConfig.name, envType: fnContext.env.envType }), fnContext);
                    return builtinResult;
                }
            }
            const result = await method(fnParams, fnContext);
            return {
                code: 0,
                data: result,
            };
        }
        catch (e) {
            const { needOriginalError: needOriginal } = options;
            if (e instanceof utils_1.TCBError) {
                return {
                    code: e.code || -1,
                    message: e.message,
                    original: needOriginal ? e : undefined,
                };
            }
            return {
                code: -1,
                message: e.message,
                original: needOriginal ? e : undefined,
            };
        }
    };
}
exports.createApp = createApp;
